@@ -749,12 +749,7 @@ class Potencia(object):
         return None
 
 
-class PeriodoExcesoPotencia(object):
-
-    def __init__(self, data, name):
-        self.periodo = data
-        self._name = name
-
+class PeriodoExcesoPotencia(Periodo):
     @property
     def valor_exceso_potencia(self):
         if hasattr(self.periodo, 'ValorExcesoPotencia'):
@@ -769,22 +764,52 @@ class PeriodoExcesoPotencia(object):
     def nombre(self):
         return self._name
 
+    def es_facturable(self):
+        """Algunas empresas envian periodos que no se deben facturar.
+        Esos tienen precio 0. Pese a eso, si tienen cantidad los facturaremos
+        igual ya que tambien hay empresas que quieren facturar lineas pero dejan
+        el precio a 0"""
+        return bool(self.valor_exceso_potencia)
+
 
 class ExcesoPotencia(object):
 
+    PERIODO_TYPE = PeriodoExcesoPotencia
+
     def __init__(self, data):
         self.exceso_potencia = data
 
     @property
     def periodos(self):
         data = []
+        periodes_no_facturables = []
         if hasattr(self.exceso_potencia, 'Periodo'):
-            period_name = 1
+            period_number = 1
+            max_facturat = period_number
 
             for d in self.exceso_potencia.Periodo:
-                period = PeriodoExcesoPotencia(d, 'P{0}'.format(period_name))
-                data.append(period)
-                period_name += 1
+                period_name = 'P{0}'.format(period_number)
+                period = self.PERIODO_TYPE(
+                    d, period_name
+                )
+                if period.es_facturable():
+                    data.append(period)
+                    max_facturat = period_number
+                else:
+                    periodes_no_facturables.append((d, period_number))
+                period_number += 1
+
+            if periodes_no_facturables:
+                max_no_facturat = max([x[1] for x in periodes_no_facturables])
+                # Per les 6.1 ens envien periodes amb preu i quantitat 0 pero si que els hem de gestionar
+                if max_facturat > max_no_facturat:
+                    for d, period_number in periodes_no_facturables:
+                        period_name = 'P{0}'.format(period_number)
+                        period = self.PERIODO_TYPE(
+                            d, period_name
+                        )
+                        data.append(period)
+
         return data
 
     @property
@@ -1554,6 +1579,7 @@ class ModeloAparato(object):
                 # If we don't have any type requirements or the current
                 # reading is in them
                 if not tipus or (integrador.tipus and integrador.tipus in tipus):
+                    integrador.comptador = self
                     lectures.append(integrador)
         except AttributeError:
             pass
@@ -1796,8 +1822,10 @@ class FacturaATR(Factura):
 
         res = []
         for t in tipus_lectures:
-            lectures_amb_ajustos = self.get_lectures_amb_periodes_td(lectures, t)
-            res.extend(lectures_amb_ajustos)
+            lectures_td = self.get_lectures_amb_periodes_td(lectures, t)
+            lectures_amb_ajustos = self.get_lectures_amb_ajust_quadrat_amb_consum(t, lectures=lectures_td, motiu_ajust='97')
+            for aux in lectures_amb_ajustos.values():
+                res.extend(aux)
         return res
 
     def get_lectures_amb_periodes_td(self, lectures, tipus):
@@ -1881,7 +1909,7 @@ class FacturaATR(Factura):
         return res
 
     def get_consum_facturat(self, tipus, periode=None):
-        if tipus not in ['A', 'S']:
+        if tipus not in ['A', 'S', 'R']:
             return None
 
         if tipus == 'A' and self.energia_activa:
@@ -1909,6 +1937,17 @@ class FacturaATR(Factura):
                 res.append(0.0)
             return res
 
+        if tipus == 'R':
+            res = []
+            if self.energia_reactiva and self.energia_reactiva.terminos_energia_reactiva:
+                for reactiva in self.energia_reactiva.terminos_energia_reactiva:
+                    for periode_reactiva in reactiva.periodos:
+                        if periode_reactiva.nombre == periode or not periode:
+                            res.append(periode_reactiva.cantidad)
+            if not res:
+                res.append(0.0)
+            return res
+
         return None
 
     def get_consums_reactiva_capacitiva_a_facturar(self):
@@ -1999,25 +2038,26 @@ class FacturaATR(Factura):
                     break
         if comptador_amb_lectures:
             base_info = comptador_amb_lectures.get_lectures_activa_entrant()[0]
-            for concepte in self.conceptos_repercutibles:
-                if concepte.concepto_repercutible[0] == '7':
-                    l1 = Lectura(None)
-                    l1.fecha = base_info.lectura_desde.fecha
-                    l1.lectura = 0
-                    l1.procedencia = base_info.lectura_desde.procedencia
-                    l2 = Lectura(None)
-                    l2.fecha = base_info.lectura_hasta.fecha
-                    l2.lectura = 0
-                    l2.procedencia = base_info.lectura_hasta.procedencia
-                    new_integrador = Integrador(None)
-                    new_integrador.magnitud = "AS"
-                    new_integrador.numero_ruedas_enteras = base_info.numero_ruedas_enteras
-                    new_integrador.codigo_periodo = base_info.codigo_periodo[0] + concepte.concepto_repercutible[1]
-                    if not new_integrador.periode:
-                        new_integrador.codigo_periodo = base_info.codigo_periodo
-                    new_integrador.lectura_desde = l1
-                    new_integrador.lectura_hasta = l2
-                    res.append(new_integrador)
+            i = 0
+            for consum in self.get_consum_facturat(tipus='S'):
+                i += 1
+                l1 = Lectura(None)
+                l1.fecha = base_info.lectura_desde.fecha
+                l1.lectura = 0
+                l1.procedencia = base_info.lectura_desde.procedencia
+                l2 = Lectura(None)
+                l2.fecha = base_info.lectura_hasta.fecha
+                l2.lectura = 0
+                l2.procedencia = base_info.lectura_hasta.procedencia
+                new_integrador = Integrador(base_info.integrador)
+                new_integrador.magnitud = "AS"
+                new_integrador.numero_ruedas_enteras = base_info.numero_ruedas_enteras
+                new_integrador.codigo_periodo = base_info.codigo_periodo[0] + str(i)
+                if not new_integrador.periode:
+                    new_integrador.codigo_periodo = base_info.codigo_periodo
+                new_integrador.lectura_desde = l1
+                new_integrador.lectura_hasta = l2
+                res.append(new_integrador)
         return res
 
     def get_fake_pX_lectura(self, tipus, periode, base_info, lectura_desde=0, lectura_hasta=0):
@@ -2084,15 +2124,27 @@ class FacturaATR(Factura):
                     res.append(new_integrador)
         return res
 
-    def get_lectures_amb_ajust_quadrat_amb_consum(self, tipus='S', ajust_balancejat=True, motiu_ajust="98"):
+    def get_lectures_amb_ajust_quadrat_amb_consum(self, tipus='S', ajust_balancejat=True, motiu_ajust="98", lectures=None):
         lectures_per_periode = {}
-        for comptador in self.get_comptadors():
-            for lectura in comptador.get_lectures(tipus):
-                lectura.comptador = comptador
-                periode = lectura.periode
-                if not lectures_per_periode.get(periode):
-                    lectures_per_periode[periode] = []
-                lectures_per_periode[periode].append(lectura)
+        lectures_del_tipus = []
+        if lectures is None:
+            for comptador in self.get_comptadors():
+                for lectura in comptador.get_lectures(tipus, force_no_transforma_no_td_a_td=True):
+                    lectura.comptador = comptador
+                    lectures_del_tipus.append(lectura)
+        else:
+            for lectura in lectures:
+                if lectura.tipus == tipus:
+                    lectures_del_tipus.append(lectura)
+
+        if not lectures_del_tipus:
+            return {}
+
+        for lectura in lectures_del_tipus:
+            periode = lectura.periode
+            if not lectures_per_periode.get(periode):
+                lectures_per_periode[periode] = []
+            lectures_per_periode[periode].append(lectura)
 
             nperiodes = len([l for l in lectures_per_periode.keys() if l])
             if self.periodes_facturats_agrupats(nperiodes, tipus=tipus) and ajust_balancejat:
@@ -2112,14 +2164,9 @@ class FacturaATR(Factura):
                 if consums_desitjats:
                     consums_repartits = repartir_consums_entre_lectures(consums_desitjats, lectures)
                     for lectura, consum in consums_repartits.iteritems():
-                        if not lectura.ajuste:
-                            lectura.ajuste = Ajuste(None)
-                        else:
-                            lectura.ajuste = Ajuste(lectura.ajuste.ajuste)
-                        old_ajust = lectura.ajuste and lectura.ajuste.ajuste_por_integrador or 0.0
-                        new_val = consum - (lectura.lectura_hasta.lectura - lectura.lectura_desde.lectura + old_ajust)
-                        if new_val != (lectura.lectura_hasta.lectura - lectura.lectura_desde.lectura + old_ajust):
-                            lectura.ajuste.ajuste_por_integrador = consum - (lectura.lectura_hasta.lectura - lectura.lectura_desde.lectura)
+                        new_val = self.get_ajust_from_consum_desitjat(lectura, consum)
+                        if new_val:
+                            lectura.ajuste.ajuste_por_integrador = new_val
                             lectura.ajuste.codigo_motivo = motiu_ajust  # normalment 98 - Autoconsumo
             for l in lectures:
                 if not res.get(l.comptador):
@@ -2127,6 +2174,39 @@ class FacturaATR(Factura):
                 res[l.comptador].append(l)
         return res
 
+    def get_ajust_from_consum_desitjat(self, lectura_a_ajustar, consum_desitjat):
+        if not lectura_a_ajustar.ajuste:
+            lectura_a_ajustar.ajuste = Ajuste(None)
+        else:
+            lectura_a_ajustar.ajuste = Ajuste(lectura_a_ajustar.ajuste.ajuste)
+
+        if lectura_a_ajustar.magnitud in ('AE', 'AS'):
+            new_val = self.get_ajust_from_consum_desitjat_Ax(lectura_a_ajustar, consum_desitjat)
+        elif lectura_a_ajustar.magnitud in ('R1', 'R2', 'R3', 'R4'):
+            new_val = self.get_ajust_from_consum_desitjat_Rx(lectura_a_ajustar, consum_desitjat)
+        else:
+            new_val = 0.0
+
+        return new_val
+
+    def get_ajust_from_consum_desitjat_Ax(self, lectura_a_ajustar, consum_desitjat):
+        new_val = consum_desitjat - (lectura_a_ajustar.lectura_hasta.lectura - lectura_a_ajustar.lectura_desde.lectura)
+        return new_val
+
+    def get_ajust_from_consum_desitjat_Rx(self, lectura_a_ajustar, consum_desitjat):
+        if consum_desitjat == 0:
+            old_ajust = lectura_a_ajustar.ajuste and lectura_a_ajustar.ajuste.ajuste_por_integrador or 0.0
+            return -(lectura_a_ajustar.lectura_hasta.lectura - lectura_a_ajustar.lectura_desde.lectura + old_ajust)
+
+        consums_facturats_activa = self.get_consum_facturat(tipus='A', periode=lectura_a_ajustar.periode)
+        consum_facturat_activa = sum(consums_facturats_activa)
+
+        reactiva_real_mesurada = consum_desitjat + (consum_facturat_activa * 0.33)
+
+        old_ajust = lectura_a_ajustar.ajuste and lectura_a_ajustar.ajuste.ajuste_por_integrador or 0.0
+        new_val = reactiva_real_mesurada - (lectura_a_ajustar.lectura_hasta.lectura - lectura_a_ajustar.lectura_desde.lectura + old_ajust)
+        return new_val
+
     def get_lectures_activa_entrant(self, ajust_balancejat=True, motiu_ajust="98"):
         return self.get_lectures_amb_ajust_quadrat_amb_consum(tipus='A', ajust_balancejat=ajust_balancejat, motiu_ajust=motiu_ajust)
 
